@@ -1,6 +1,6 @@
 /*
  * The Python Imaging Library.
- * $Id$
+ * $Id: //modules/pil/_imagingtk.c#3 $
  *
  * tkinter hooks
  *
@@ -64,11 +64,8 @@ static PyMethodDef functions[] = {
     {NULL, NULL} /* sentinel */
 };
 
-void
-#ifdef WIN32
-__declspec(dllexport)
-#endif
-init_imagingtk()
+DL_EXPORT(void)
+init_imagingtk(void)
 {
     Py_InitModule("_imagingtk", functions);
 }
